/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <gmpxx.h>
#include <mpc_class.h>

inline mpf_class mpf_sign(mpf_class a, mpf_class b)
{
  if (b>=0.0) return abs(a);
  return -abs(a);
}

#if !defined (__MPACK_ERRNO__)
extern int mpack_errno;
#endif

#if !defined (MPACK_DEFAULT_PRECISION)
#define MPACK_DEFAULT_PRECISION 256
#endif

/* Initalization */
void      mpack_initialize(int precision);

/* LEVEL 1 MBLAS */
void      Crotg   (mpc_class* ca, mpc_class cb, mpf_class* c, mpc_class* s);
void      Rrotg   (mpf_class* da, mpf_class* db, mpf_class* c, mpf_class* s);
void      Rrotmg  (mpf_class* dd1, mpf_class* dd2, mpf_class* dx1, mpf_class dy1, mpf_class* dparam);
void      Rrot    (int n, mpf_class* dx, int incx, mpf_class* dy, int incy, mpf_class c, mpf_class s);
void      Rrotm   (int n, mpf_class *x, int incx, mpf_class *y, int incy, mpf_class* dparam);
void      CRrot   (int n, mpc_class* cx, int incx, mpc_class* cy, int incy, mpf_class c, mpf_class s);
void      Cswap   (int n, mpc_class* cx, int incx, mpc_class* cy, int incy);
void      Rswap   (int n, mpf_class* dx, int incx, mpf_class* dy, int incy);
void      Cscal   (int n, mpc_class ca, mpc_class* cx, int incx);
void      CRscal  (int n, mpf_class sa, mpc_class* cx, int incx);
void      Rscal   (int n, mpf_class ca, mpf_class* cx, int incx);
void      Ccopy   (int n, mpc_class* cx, int incx, mpc_class* cy, int incy);
void      Rcopy   (int n, mpf_class* dx, int incx, mpf_class* dy, int incy);
void      Caxpy   (int n, mpc_class ca, mpc_class* cx, int incx, mpc_class* cy, int incy);
void      Raxpy   (int n, mpf_class da, mpf_class* dx, int incx, mpf_class* dy, int incy);
mpf_class Rdot    (int n, mpf_class* dx, int incx, mpf_class* dy, int incy);
mpc_class Cdotc   (int n, mpc_class* cx, int incx, mpc_class* cy, int incy);
mpc_class Cdotu   (int n, mpc_class* cx, int incx, mpc_class* cy, int incy);
mpf_class RCnrm2  (int n, mpc_class* x, int incx);
mpf_class Rnrm2   (int n, mpf_class* x, int incx);
mpf_class RCasum  (int n, mpc_class* zx, int incx);
mpf_class Rasum   (int n, mpf_class* dx, int incx);
int       iCamax  (int n, mpc_class* dx, int incx);
int       iRamax  (int n, mpf_class* dx, int incx);
mpf_class RCabs1  (mpc_class z);
int       Mlsame  (const char* a, const char* b);
void      Mxerbla (const char* srname, int info);

/* LEVEL 2 MBLAS */
void      Cgemv   (const char *trans, int m, int n, mpc_class alpha, mpc_class *A, int lda, mpc_class *x, int incx, mpc_class beta, mpc_class *y, int incy);
void      Rgemv   (const char *trans, int m, int n, mpf_class alpha, mpf_class *A,int lda, mpf_class *x, int incx, mpf_class beta, mpf_class *y, int incy);
void      Cgbmv   (const char *trans, int m, int n, int kl, int ku, mpc_class alpha, mpc_class* A, int lda, mpc_class* x, int incx, mpc_class beta, mpc_class *y, int incy);
void      Rgbmv   (const char* trans, int m, int n, int kl, int ku, mpf_class alpha, mpf_class* A, int lda, mpf_class* x, int incx, mpf_class beta, mpf_class* y, int incy);
void      Chemv   (const char* uplo, int n, mpc_class alpha, mpc_class* A, int lda, mpc_class* x, int incx, mpc_class beta, mpc_class* y, int incy);
void      Chbmv   (const char* uplo, int n, int k, mpc_class alpha, mpc_class* A, int lda, mpc_class* x, int incx, mpc_class beta, mpc_class* y, int incy);
void      Chpmv   (const char* uplo, int n, mpc_class alpha, mpc_class* AP, mpc_class* x, int incx, mpc_class beta, mpc_class* y, int incy);
void      Rsymv   (const char* uplo, int n, mpf_class alpha, mpf_class* A, int lda, mpf_class* x, int incx, mpf_class beta, mpf_class* y, int incy);
void      Rsbmv   (const char* uplo, int n, int k, mpf_class alpha, mpf_class* A, int lda, mpf_class* x, int incx, mpf_class beta, mpf_class* y, int incy);
void      Rspmv   (const char* uplo, int n, mpf_class alpha, mpf_class* AP, mpf_class* x, int incx, mpf_class beta, mpf_class* y, int incy);
void      Ctrmv   (const char* uplo, const char* trans, const char* diag, int n, mpc_class* A, int lda, mpc_class* x, int incx);
void      Rtrmv   (const char* uplo, const char* trans, const char* diag, int n, mpf_class* A, int lda, mpf_class* x, int incx);
void      Ctbmv   (const char* uplo, const char* trans, const char* diag, int n, int k, mpc_class* A, int lda, mpc_class* x, int incx);
void      Rtbmv   (const char* uplo, const char* trans, const char* diag, int n, int k, mpf_class* A, int lda, mpf_class* x, int incx);
void      Ctpmv   (const char* uplo, const char* trans, const char* diag, int n, mpc_class* AP, mpc_class* x, int incx);
void      Rtpmv   (const char* uplo, const char* trans, const char* diag, int n, mpf_class* AP, mpf_class* x, int incx);
void      Ctrsv   (const char* uplo, const char* trans, const char* diag, int n, mpc_class* A, int lda, mpc_class* x, int incx);
void      Rtrsv   (const char* uplo, const char* trans, const char* diag, int n, mpf_class* A, int lda, mpf_class* x, int incx);
void      Ctbsv   (const char* uplo, const char* trans, const char* diag, int n, int k, mpc_class* A, int lda, mpc_class* x, int incx);
void      Rtbsv   (const char *uplo, const char *trans, const char *diag, int n, int k, mpf_class *A, int lda, mpf_class *x, int incx);
void      Ctpsv   (const char* uplo, const char* trans, const char* diag, int n, mpc_class* AP, mpc_class* x, int incx);
void      Rtpsv   (const char *uplo, const char *trans, const char *diag, int n, mpf_class *AP, mpf_class *x, int incx);
void      Rger    (int m, int n, mpf_class alpha, mpf_class* x, int incx, mpf_class* y, int incy, mpf_class* A, int lda);
void      Cgeru   (int m, int n, mpc_class alpha, mpc_class* x, int incx, mpc_class* y, int incy, mpc_class* A, int lda);
void      Cgerc   (int m, int n, mpc_class alpha, mpc_class* x, int incx, mpc_class* y, int incy, mpc_class* A, int lda);
void      Cher    (const char* uplo, int n, mpf_class alpha, mpc_class* x, int incx, mpc_class* A, int lda);
void      Chpr    (const char* uplo, int n, mpf_class alpha, mpc_class* x, int incx, mpc_class* AP);
void      Cher2   (const char* uplo, int n, mpc_class alpha, mpc_class* x, int incx, mpc_class* y, int incy, mpc_class* A, int lda);
void      Chpr2   (const char* uplo, int n, mpc_class alpha, mpc_class* x, int incx, mpc_class* y, int incy, mpc_class* AP);
void      Rsyr    (const char* uplo, int n, mpf_class alpha, mpf_class* x, int incx, mpf_class* A, int lda);
void      Rspr    (const char* uplo, int n, mpf_class alpha, mpf_class* x, int incx, mpf_class* AP);
void      Rsyr2   (const char *uplo, int n, mpf_class alpha, mpf_class *x, int incx, mpf_class *y, int incy, mpf_class *A, int lda);
void      Rspr2   (const char* uplo, int n, mpf_class alpha, mpf_class* x, int incx, mpf_class* y, int incy, mpf_class* AP);

/* LEVEL 3 MBLAS */
void      Cgemm   (const char* transa, const char* transb, int m, int n, int k, mpc_class alpha, mpc_class* A,int lda, mpc_class* B, int ldb, mpc_class beta, mpc_class* C, int ldc);
void      Rgemm (const char *transa, const char *transb, int m, int n, int k, mpf_class alpha, mpf_class *A,int lda, mpf_class *B, int ldb, mpf_class beta, mpf_class *C, int ldc);
void      Csymm   (const char* side, const char* uplo, int m, int n, mpc_class alpha, mpc_class* A, int lda, mpc_class* B, int ldb, mpc_class beta, mpc_class* C, int ldc);
void      Rsymm   (const char* side, const char* uplo, int m, int n, mpf_class alpha, mpf_class* A, int lda, mpf_class* B, int ldb, mpf_class beta, mpf_class* C, int ldc);
void      Chemm   (const char* side, const char* uplo, int m, int n, mpc_class alpha, mpc_class* A, int lda, mpc_class* B, int ldb, mpc_class beta, mpc_class* C, int ldc);
void      Csyrk  (const char* uplo, const char* trans, int n, int k, mpc_class alpha, mpc_class* A, int lda, mpc_class beta, mpc_class* C, int ldc);
void      Rsyrk   (const char* uplo, const char* trans, int n, int k, mpf_class alpha, mpf_class* A, int lda, mpf_class beta, mpf_class* C, int ldc);
void      Cherk   (const char* uplo, const char* trans, int n, int k, mpf_class alpha, mpc_class* A, int lda, mpf_class beta, mpc_class* C, int ldc);
void      Csyr2k  (const char* uplo, const char* trans, int n, int k, mpc_class alpha, mpc_class* A, int lda, mpc_class* B, int ldb, mpc_class beta, mpc_class* C, int ldc);
void      Rsyr2k  (const char* uplo, const char* trans, int n, int k, mpf_class alpha, mpf_class* A, int lda, mpf_class* B, int ldb, mpf_class beta, mpf_class* C, int ldc);
void      Cher2k  (const char* uplo, const char* trans, int n, int k, mpc_class alpha, mpc_class* A, int lda, mpc_class* B, int ldb, mpf_class beta, mpc_class* C, int ldc);
void      Ctrmm   (const char* side, const char* uplo, const char* transa, const char* diag, int m, int n, mpc_class alpha, mpc_class* A, int lda, mpc_class* B, int ldb);
void      Rtrmm   (const char* side, const char* uplo, const char* transa, const char* diag, int m, int n, mpf_class alpha, mpf_class* A, int lda, mpf_class* B, int ldb);
void      Ctrsm   (const char* side, const char* uplo, const char* transa, const char* diag, int m, int n, mpc_class alpha, mpc_class* A, int lda, mpc_class* B, int ldb);
void      Rtrsm   (const char* side, const char* uplo, const char* transa, const char* diag, int m, int n, mpf_class alpha, mpf_class* A, int lda, mpf_class* B, int ldb);
